@@ -4,7 +4,7 @@
  * This program is distributed under the GNU General Public License, version
  * 2.1. A copy of this license is included with this source.
  *
- * Copyright (C) 2002 Gian-Carlo Pascutto and Magnus Holmgren
+ * Copyright (C) 2002, 2004 Gian-Carlo Pascutto and Magnus Holmgren
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
 #include "gain_analysis.h"
@@ -26,10 +27,18 @@
 #ifdef WIN32
 #include <io.h>
 #include <sys/utime.h>
-#else
+#else /* #ifdef WIN32 */
 #include <utime.h>
+#endif /* #ifdef WIN32 */
+
+
+#ifndef MIN
+#define MIN(x,y) ((x) < (y) ? (x) : (y))
 #endif
 
+#ifndef MAX
+#define MAX(x,y) ((x) > (y) ? (x) : (y))
+#endif
 
 #define TAG_TRACK_GAIN     "REPLAYGAIN_TRACK_GAIN"
 #define TAG_TRACK_PEAK     "REPLAYGAIN_TRACK_PEAK"
@@ -43,6 +52,10 @@
 #define PEAK_FORMAT "%1.8f"
 #define GAIN_FORMAT "%+2.2f dB"
 #define PROGRESS_FORMAT " %3d%% - %s\r"
+/* Size of PROGRESS_FORMAT when printed, excluding the filename */
+#define PROGRESS_FORMAT_SIZE 8
+#define MIN_FILENAME_SIZE 5
+#define MIN_MIDDLE_TRUNCATE_SIZE 20
 #define TEMP_NAME "vorbisgain.tmp"
 
 
@@ -208,7 +221,7 @@ int get_old_tags(const char* filename, int album_tag, float* track_peak,
                     }
                 }
 
-                if(!tag_compare(vc->user_comments[i], TAG_ALBUM_GAIN_OLD))
+                if (!tag_compare(vc->user_comments[i], TAG_ALBUM_GAIN_OLD))
                 {
                     if (sscanf(&vc->user_comments[i][sizeof(TAG_ALBUM_GAIN_OLD)],
                         "%f", album_gain) == 1)
@@ -240,6 +253,65 @@ int get_old_tags(const char* filename, int album_tag, float* track_peak,
     return result;
 }
 
+/**
+ * \brief Show/update a simple progress bar for a file.
+ *
+ * Show/update a simple progress bar for a file. The name will be truncated 
+ * to fit within the current console's with, if possible.
+ *
+ * \param vf         Vorbis file being processed.
+ * \param filename   name of the file being processed.
+ * \param position   position in the file, in percent (0-100).
+ */
+void show_progress(const char* filename, unsigned int position)
+{
+    unsigned int width;
+    unsigned int height;
+    char* short_name = NULL;
+    const char* name = filename;
+
+    if (!get_console_size(1, &width, &height))
+    {
+        unsigned int full_length = strlen(filename);
+
+        if (full_length + PROGRESS_FORMAT_SIZE > width)
+        {
+            /* Need room for cursor as well. */
+            int short_length = width - PROGRESS_FORMAT_SIZE - 1;
+            
+            /* Not useful to cut it too short. */
+            short_length = MAX(short_length, MIN_FILENAME_SIZE + 3);
+            short_name = malloc(short_length + 1);
+            /* Need space for "..." */
+            short_length -= 3;
+
+            /* Put "..." in the middle if we have "enough" chars available. */
+            if (short_length > MIN_MIDDLE_TRUNCATE_SIZE)
+            {
+                strncpy(short_name, filename, short_length / 2);
+                short_name[short_length / 2] = '\0';
+                strcat(short_name, "...");
+                /* Round upwards here, so that all chars are used. */
+                strcat(short_name, &filename[full_length 
+                    - ((short_length + 1) / 2)]);
+            }
+            else
+            {
+                strcpy(short_name, "...");
+                strcat(short_name, &filename[full_length - short_length]);
+            }
+
+            name = short_name;
+        }
+    }
+
+    fprintf(stderr, _(PROGRESS_FORMAT), MIN(position, 100), name);
+
+    if (NULL != short_name)
+    {
+        free(short_name);
+    }
+}
 
 /**
  * Get the replaygain and peak value for a file.
@@ -263,6 +335,7 @@ int get_gain(const char* filename, float* track_peak, float* track_gain,
     float new_peak;
     long rate;
     int previous_percentage = -1;
+    time_t previous_time = 0;
     int previous_section = 0;
     int current_section;
     int channels;
@@ -331,7 +404,7 @@ int get_gain(const char* filename, float* track_peak, float* track_gain,
 
         if ((file_length > 0) && (ov_pcm_tell(&vf) >= 0))
         {
-            fprintf(stderr, PROGRESS_FORMAT, 0, filename);
+            show_progress(filename, 0);
         }
     }
 
@@ -427,11 +500,16 @@ int get_gain(const char* filename, float* track_peak, float* track_gain,
             if (file_pos > 0)
             {
                 int percentage = (int) floor((((double) file_pos / file_length) * 100) + 0.5);
+                time_t curr_time;
 
-                if (percentage != previous_percentage)
+                time(&curr_time);
+
+                if ((100 == percentage) || ((percentage != previous_percentage) 
+                    && (difftime(curr_time, previous_time) >= 1)))
                 {
-                    fprintf(stderr, _(PROGRESS_FORMAT), percentage, filename);
+                    show_progress(filename, percentage);
                     previous_percentage = percentage;
+                    previous_time = curr_time;
                 }
             }
         }
@@ -571,18 +649,7 @@ int write_gains(const char *filename, float track_peak, float track_gain,
     /* Add the old tags to the new file */
     for (i = 0; i < new_count; i++)
     {
-        char* splitter = (char*) strchr(new_tags[i], '=');
-
-        if (splitter != NULL)
-        {
-            *splitter++ = '\0';
-            vorbis_comment_add_tag(vc, new_tags[i], splitter);
-        }
-        else
-        {
-            fprintf(stderr, _("Malformed (?) tag in '%s' skipped ('%s')\n"),
-                filename, new_tags[i]);
-        }
+        vorbis_comment_add(vc, new_tags[i]);
     }
 
     /* Add new tags - unless ReplayGain tags are to be removed */
